Package: stochtree
Title: Stochastic Tree Ensembles (XBART and BART) for Supervised
        Learning and Causal Inference
Version: 0.4.5
Authors@R: 
  c(
    person("Drew", "Herren", email = "drewherrenopensource@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0003-4109-6611")), 
    person("Richard", "Hahn", role = "aut"),
    person("Jared", "Murray", role = "aut"),
    person("Carlos", "Carvalho", role = "aut"),
    person("Jingyu", "He", role = "aut"),
    person("Pedro", "Lima", role = "ctb"),
    person("Entejar", "Alam", role = "ctb"),
    person("stochtree", "contributors", role = c("cph")),
    person("Eigen", "contributors", role = c("cph"), comment = "C++ source uses the Eigen library for matrix operations, see inst/COPYRIGHTS"),
    person("xgboost", "contributors", role = c("cph"), comment = "C++ tree code and related operations include or are inspired by code from the xgboost library, see inst/COPYRIGHTS"),
    person("treelite", "contributors", role = c("cph"), comment = "C++ tree code and related operations include or are inspired by code from the treelite library, see inst/COPYRIGHTS"),
    person("Microsoft", "Corporation", role = c("cph"), comment = "C++ I/O and various project structure code include or are inspired by code from the LightGBM library, which is a copyright of Microsoft, see inst/COPYRIGHTS"),
    person("Niels", "Lohmann", role = c("cph"), comment = "C++ source uses the JSON for Modern C++ library for JSON operations, see inst/COPYRIGHTS"),
    person("Daniel", "Lemire", role = c("cph"), comment = "C++ source uses the fast_double_parser library internally, see inst/COPYRIGHTS"),
    person("Victor", "Zverovich", role = c("cph"), comment = "C++ source uses the fmt library internally, see inst/COPYRIGHTS")
  )
Copyright: Copyright details for stochtree's C++ dependencies, which
        are vendored along with the core stochtree source code, are
        detailed in inst/COPYRIGHTS
Description: Flexible stochastic tree ensemble software. 
    Robust implementations of Bayesian Additive Regression Trees (BART) 
    (Chipman, George, McCulloch (2010) <doi:10.1214/09-AOAS285>) 
    for supervised learning and Bayesian Causal Forests (BCF) 
    (Hahn, Murray, Carvalho (2020) <doi:10.1214/19-BA1195>) 
    for causal inference. Enables model serialization and parallel sampling 
    and provides a low-level interface for custom stochastic forest samplers.
    Includes the grow-from-root algorithm for accelerated forest sampling 
    (He and Hahn (2021) <doi:10.1080/01621459.2021.1942012>), 
    a log-linear leaf model for forest-based heteroskedasticity (Murray (2020) <doi:10.1080/01621459.2020.1813587>), 
    and the cloglog BART model of Alam and Linero (2025) <doi:10.48550/arXiv.2502.00606> for ordinal outcomes.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
LinkingTo: cpp11, BH
Suggests: testthat (>= 3.0.0),
SystemRequirements: C++17
Imports: R6, stats
URL: https://stochtree.ai/, https://github.com/StochasticTree/stochtree
BugReports: https://github.com/StochasticTree/stochtree/issues
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-06-30 14:51:01 UTC; drew
Author: Drew Herren [aut, cre] (ORCID: <https://orcid.org/0000-0003-4109-6611>),
  Richard Hahn [aut],
  Jared Murray [aut],
  Carlos Carvalho [aut],
  Jingyu He [aut],
  Pedro Lima [ctb],
  Entejar Alam [ctb],
  stochtree contributors [cph],
  Eigen contributors [cph] (C++ source uses the Eigen library for matrix
    operations, see inst/COPYRIGHTS),
  xgboost contributors [cph] (C++ tree code and related operations
    include or are inspired by code from the xgboost library, see
    inst/COPYRIGHTS),
  treelite contributors [cph] (C++ tree code and related operations
    include or are inspired by code from the treelite library, see
    inst/COPYRIGHTS),
  Microsoft Corporation [cph] (C++ I/O and various project structure code
    include or are inspired by code from the LightGBM library, which is
    a copyright of Microsoft, see inst/COPYRIGHTS),
  Niels Lohmann [cph] (C++ source uses the JSON for Modern C++ library
    for JSON operations, see inst/COPYRIGHTS),
  Daniel Lemire [cph] (C++ source uses the fast_double_parser library
    internally, see inst/COPYRIGHTS),
  Victor Zverovich [cph] (C++ source uses the fmt library internally, see
    inst/COPYRIGHTS)
Maintainer: Drew Herren <drewherrenopensource@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-30 16:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-30 17:25:43 UTC; unix
Archs: stochtree.so.dSYM
