# Generated by roxygen2: do not edit by hand

S3method(extractParameter,bartmodel)
S3method(extractParameter,bcfmodel)
S3method(getRandomEffectSamples,bartmodel)
S3method(getRandomEffectSamples,bcfmodel)
S3method(plot,bartmodel)
S3method(plot,bcfmodel)
S3method(predict,bartmodel)
S3method(predict,bcfmodel)
S3method(print,ForestSamples)
S3method(print,RandomEffectSamples)
S3method(print,bartmodel)
S3method(print,bcfmodel)
S3method(summary,bartmodel)
S3method(summary,bcfmodel)
export(OutcomeModel)
export(bart)
export(bcf)
export(calibrateInverseGammaErrorVariance)
export(computeBARTPosteriorInterval)
export(computeBCFPosteriorInterval)
export(computeContrastBARTModel)
export(computeContrastBCFModel)
export(computeForestLeafIndices)
export(computeForestLeafVariances)
export(computeForestMaxLeafIndex)
export(createBARTModelFromCombinedJson)
export(createBARTModelFromCombinedJsonString)
export(createBARTModelFromJson)
export(createBARTModelFromJsonFile)
export(createBARTModelFromJsonString)
export(createBCFModelFromCombinedJson)
export(createBCFModelFromCombinedJsonString)
export(createBCFModelFromJson)
export(createBCFModelFromJsonFile)
export(createBCFModelFromJsonString)
export(createCppJson)
export(createCppJsonFile)
export(createCppJsonString)
export(createCppRNG)
export(createForest)
export(createForestDataset)
export(createForestModel)
export(createForestModelConfig)
export(createForestSamples)
export(createGlobalModelConfig)
export(createOutcome)
export(createPreprocessorFromJson)
export(createPreprocessorFromJsonString)
export(createRandomEffectSamples)
export(createRandomEffectsDataset)
export(createRandomEffectsModel)
export(createRandomEffectsTracker)
export(extractParameter)
export(getRandomEffectSamples)
export(loadForestContainerCombinedJson)
export(loadForestContainerCombinedJsonString)
export(loadForestContainerJson)
export(loadRandomEffectSamplesCombinedJson)
export(loadRandomEffectSamplesCombinedJsonString)
export(loadRandomEffectSamplesJson)
export(loadScalarJson)
export(loadVectorJson)
export(preprocessPredictionData)
export(preprocessTrainData)
export(resetActiveForest)
export(resetForestModel)
export(resetRandomEffectsModel)
export(resetRandomEffectsTracker)
export(rootResetRandomEffectsModel)
export(rootResetRandomEffectsTracker)
export(sampleBARTPosteriorPredictive)
export(sampleBCFPosteriorPredictive)
export(sampleGlobalErrorVarianceOneIteration)
export(sampleLeafVarianceOneIteration)
export(sample_without_replacement)
export(saveBARTModelToJson)
export(saveBARTModelToJsonFile)
export(saveBARTModelToJsonString)
export(saveBCFModelToJson)
export(saveBCFModelToJsonFile)
export(saveBCFModelToJsonString)
export(savePreprocessorToJson)
export(savePreprocessorToJsonString)
importFrom(R6,R6Class)
importFrom(stats,coef)
importFrom(stats,dnorm)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qgamma)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,resid)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,sigma)
importFrom(stats,var)
useDynLib(stochtree, .registration = TRUE)
