# stochtree R package

Software for building stochastic tree ensembles (i.e. BART, XBART) for supervised learning and causal inference.

## Getting started

`stochtree` can be installed from CRAN via

```
install.packages("stochtree")
```

The development version of stochtree can be installed from github via

```
remotes::install_github("StochasticTree/stochtree", ref="r-dev")
```
