## usethis namespace: start
#' @importFrom stats coef
#' @importFrom stats dnorm
#' @importFrom stats lm
#' @importFrom stats model.matrix
#' @importFrom stats predict
#' @importFrom stats qgamma
#' @importFrom stats qnorm
#' @importFrom stats quantile
#' @importFrom stats pnorm
#' @importFrom stats rbinom
#' @importFrom stats resid
#' @importFrom stats rnorm
#' @importFrom stats runif
#' @importFrom stats sd
#' @importFrom stats sigma
#' @importFrom stats var
#' @importFrom R6 R6Class
## usethis namespace: end
NULL

#' @useDynLib stochtree, .registration = TRUE
"_PACKAGE"
