% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bart.R
\name{summary.bartmodel}
\alias{summary.bartmodel}
\title{Summarize BART Model Fit and Parameters}
\usage{
\method{summary}{bartmodel}(object, ...)
}
\arguments{
\item{object}{The BART model object}

\item{...}{Additional arguments}
}
\value{
BART model object unchanged after summarizing
}
\description{
Summarize a BART fit with a description of the model that was fit and numeric summaries of any sampled quantities.
}
